@@ -87,6 +87,16 @@ class Key(object):
       self._list = map(Namespace, self._string.split('/'))
     return self._list
 
+  @property
+  def reverse(self):
+    '''Returns the reverse of this Key.
+
+        >>> Key('/Comedy/MontyPython/Actor:JohnCleese').reverse
+        Key('/Actor:JohnCleese/MontyPython/Comedy')
+
+    '''
+    return Key(self.list[::-1])
+
   @property
   def namespaces(self):
     '''Returns the list of namespaces of this Key.'''
@@ -102,6 +112,11 @@ class Key(object):
     '''Returns the type of this Key, the field of the last namespace.'''
     return Namespace(self.list[-1]).field
 
+  def instance(self, other):
+    '''Returns an instance Key, by appending a name to the namespace.'''
+    assert '/' not in str(other)
+    return Key(str(self) + ':' + str(other))
+
   @property
   def path(self):
     '''Returns the path of this Key, the parent and the type.'''
@@ -119,16 +134,6 @@ class Key(object):
       return Key(self.list[:-1])
     raise ValueError('%s is base key (it has no parent)' % repr(self))
 
-  @property
-  def reverse(self):
-    '''Returns the reverse of this Key.
-
-        >>> Key('/Comedy/MontyPython/Actor:JohnCleese').reverse
-        Key('/Actor:JohnCleese/MontyPython/Comedy')
-
-    '''
-    return Key(self.list[::-1])
-
   def child(self, other):
     '''Returns the child Key by appending namespace `other`.
 
